<?php

declare(strict_types=1);

namespace Dedi\SyliusCmsPlugin\Form\Type\UiElement;

use Dedi\SyliusCmsPlugin\Form\Type\UiElement\Base\LinkType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\TextType as FormTextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Validator\Constraints\Count;

class ProductCollectionType extends AbstractUiElementType
{
    public function __construct(private int $collectionCountMin, private int $collectionCountMax)
    {
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('title', FormTextType::class, [
                'label' => 'dedi_cms.ui_element.elements.title',
                'required' => false,
            ])
            ->add('htmlTag', ChoiceType::class, [
                'label' => 'dedi_cms.ui_element.elements.html_tag',
                'choices' => [
                    'H1' => 'h1',
                    'H2' => 'h2',
                    'H3' => 'h3',
                    'H4' => 'h4',
                    'H5' => 'h5',
                    'H6' => 'h6',
                    'P' => 'p',
                ],
                'empty_data' => 'h2',
            ])
            ->add('link', LinkType::class, [
                'label' => false,
                'required' => false,
                'embedded' => true,
            ])
            ->add('collection', CollectionType::class, [
            'entry_type' => ProductType::class,
            'entry_options' => [
                'embedded' => true,
                'label' => false,
            ],
            'button_add_label' => 'dedi_cms.ui_element.product_collection.add',
            'allow_add' => true,
            'prototype' => true,
            'allow_delete' => true,
            'by_reference' => false,
            'delete_empty' => true,
            'label' => 'dedi_cms.ui_element.product_collection.collection',
            'constraints' => [
                new Count([
                    'min' => $this->collectionCountMin,
                    'max' => $this->collectionCountMax,
                ]),
            ],
        ]);
    }
}
